<?php
namespace Ma\Web;

use Exception;

/**
 * @package         Ma\Web
 * @class           SessionManager
 * @author          Mahdi Abu Yasmine
 * @date            2024-02-17 04:09:12 CET
 * @copyright       meta-tech.academy
 */
class SessionManager
{
    protected array $config;

    private const DURATION    = 180;
    private const ALGO        = 'sha256';
    private const SALT        = "xL5!b8<àe3':oP";
    private const PREFIX_NAME = "SESS";

    /**
     * @public
     * @constructor
     * @param       array   $string
     * @return      string
     */
    public function __construct(array $config = [])
    {
        if (!isset($config['duration'])) {
            $config['duration'] = self::DURATION;
        }
        if (!isset($config['salt'])) {
            $config['salt'] = self::SALT;
        }
        if (!isset($config['algo'])) {
            $config['algo'] = self::ALGO;
        }
        if (!isset($config['prefix'])) {
            $config['prefix'] = self::PREFIX_NAME;
        }
        $this->config = $config;
    }

    /**
     * @protected
     * @return      string
     */
    protected function getUserId(bool $short=false): string
    {
        $id = hash($this->config['algo'], $this->config['salt'] . $this->getUserAgent() . $this->getUserIp());
        return $short ? substr($id, -9) : $id;
    }

    /**
     * @public
     * @return     ?string
     */
    public function getUserAgent(): ?string
    {
        return $_SERVER['HTTP_USER_AGENT']?: null;
    }

    /**
     * @public
     * @return      string
     */
    public function getUserIp(): string
    {
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        return $ip;
    }

    /**
     * @public
     * @param       bool    $start
     */
    public function start($force=false)
    {
        ini_set('session.use_strict_mode', 1);
        session_name($this->config['prefix'].$this->getUserId());
        session_start();
        if ($force || (empty($_SESSION['deleted_time']) || $_SESSION['deleted_time'] < time() - self::DURATION)) {
            session_destroy();
            $this->regenId();
        }
    }

    /**
     * @public
     */
    public function regenId()
    {
        if (session_status() != PHP_SESSION_ACTIVE) {
            session_name($this->config['prefix'].$this->getUserId());
            session_start();
        }
        $_SESSION['deleted_time'] = time();
        $_SESSION['killed_time'] = $_SESSION['deleted_time'] + self::DURATION;
        session_commit();
        ini_set('session.use_strict_mode', 0);
        session_create_id($this->getUserId(true));
        session_name($this->config['prefix'].$this->getUserId());
        session_start();
        ini_set('session.use_strict_mode', 1);
    }

    /**
     * @public
     * @param       string  $name
     * @param       mixed   $data
     */
    public function set(string $name, $data)
    {
        $oval = @$_SESSION[$name];
        $_SESSION[$name] = $data;
        if ($oval != $data) {
            $this->refresh();
        }
    }

    /**
     * @public
     */
    public function refresh()
    {
        $_SESSION['deleted_time'] = time();
        $_SESSION['killed_time'] = $_SESSION['deleted_time'] + self::DURATION;
    }

    /**
     * @public
     * @return      string
     */
    public function get(string $name): string
    {
        return $_SESSION[$name];
    }
}
